﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using System.ComponentModel;
using System.Diagnostics;

namespace XamGridHierarchicalDT
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
    }

    public class ViewModel : INotifyPropertyChanged
    {
        DataView view;

        public ViewModel()
        {
            GridSource = GenerateData().Tables[0].DefaultView;
        }

        public DataView GridSource
        {
            get { return view; }
            set
            {
                view = null;
                OnPropertyChanged("GridSource");
                view = value;
                OnPropertyChanged("GridSource");
                
            }
        }

        public ICommand ChangeDataCommand
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>((o) =>
                    {
                        GridSource = GenerateData1().Tables[0].DefaultView;
                    }));
            }
        }

        
        private DataSet GenerateData()
        {
            var data = new DataSet("SimpleData");
        //   data.EnforceConstraints = false;

            DataTable parent = new DataTable("Parent");

            parent.Columns.Add("column1", typeof(int));
            parent.Columns.Add("column2", typeof(int));
            parent.Columns.Add("column3", typeof(int));

            parent.PrimaryKey = new[] { parent.Columns[0] };

            DataTable children = new DataTable("Children");

            children.Columns.Add("column1", typeof(int));
            children.Columns.Add("column4", typeof(int));
            children.Columns.Add("column5", typeof(int));;

          //  children.PrimaryKey = new[] { children.Columns[0] };

            data.Tables.Add(parent);
            data.Tables.Add(children);


            data.Relations.Add("Parent_Child", parent.Columns[0], children.Columns[0], false);
            Random r = new Random();
            for (int i = 1; i < 4; i++)
            {

                data.Tables["Parent"].Rows.Add(new object[] { i, r.Next(100), r.Next(100) });
                for (int j = 0; j < 5; j++)
                {
                    data.Tables["Children"].Rows.Add(new object[] {i, r.Next(100), r.Next(100) });
                }
            }

            data.AcceptChanges();

            return data;
        }

        private DataSet GenerateData1()
        {
            var data = new DataSet("SimpleData");
            //   data.EnforceConstraints = false;

            DataTable parent = new DataTable("Parent");

            parent.Columns.Add("column1", typeof(int));
            parent.Columns.Add("column7", typeof(int));
            parent.Columns.Add("column8", typeof(int));

            parent.PrimaryKey = new[] { parent.Columns[0] };

            DataTable children = new DataTable("Children");

            children.Columns.Add("column1", typeof(int));
            children.Columns.Add("column9", typeof(int));
            children.Columns.Add("column10", typeof(int)); ;

            //  children.PrimaryKey = new[] { children.Columns[0] };

            data.Tables.Add(parent);
            data.Tables.Add(children);


            data.Relations.Add("Parent_Child", parent.Columns[0], children.Columns[0], false);
            Random r = new Random();
            for (int i = 1; i < 4; i++)
            {

                data.Tables["Parent"].Rows.Add(new object[] { i, r.Next(100), r.Next(100) });
                for (int j = 0; j < 5; j++)
                {
                    data.Tables["Children"].Rows.Add(new object[] { i, r.Next(100), r.Next(100) });
                }
            }

            data.AcceptChanges();

            return data;
       }


        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

    }



    public class RelayCommand<T> : ICommand
    {

        #region Declarations

        readonly Predicate<T> _canExecute;
        readonly Action<T> _execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested += value;
            }
            remove
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested -= value;
            }
        }

        [DebuggerStepThrough]
        public virtual Boolean CanExecute(Object parameter)
        {
            return _canExecute == null ? true : _canExecute((T)parameter);
        }

        public virtual void Execute(Object parameter)
        {
            _execute((T)parameter);
        }

        #endregion
    }
}
